# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: install.py 783 2010-01-09 14:28:06Z icemac $
"""Initial generation."""


def evolve(context):
    # nothing to install initially
    pass



from icemac.addressbook.i18n import _
import icemac.addressbook.browser.base
import z3c.authviewlet.session
import zope.pluggableauth.plugins.session


class FlashedSessionCredentialsPlugin(
    zope.pluggableauth.plugins.session.SessionCredentialsPlugin,
    icemac.addressbook.browser.base.FlashView):
    """SessionCredentialsPlugin with support for flash messages."""

    def challenge(self, request):
        result = super(FlashedSessionCredentialsPlugin,
                       self).challenge(request)
        if result:
            self.send_flash(
                _('To log-in enter your username and password and submit the '
                  'form.'))
        return result


class FlashedSessionCredentialsLoginForm(
    z3c.authviewlet.session.SessionCredentialsLoginForm,
    icemac.addressbook.browser.base.FlashView):
    """SessionCredentialsLoginForm with flash messages."""

    def update(self):
        super(FlashedSessionCredentialsLoginForm, self).update()
        if str(self.request.response.getStatus()).startswith('3'):
            self.send_flash(_('You have been logged-in successfully.'))
        elif 'SUBMIT' in self.request:
            self.send_flash(
                _('Login failed. Username and/or password might be wrong.'))



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

import BTrees.Length
import icemac.addressbook.namechooser.interfaces
import persistent
import zope.component
import zope.container.contained
import zope.interface


class NameSuffix(persistent.Persistent, zope.container.contained.Contained):
    "Storage for name suffix."

    zope.interface.implements(
        icemac.addressbook.namechooser.interfaces.INameSuffix)
    zope.component.adapts(
        icemac.addressbook.namechooser.interfaces.IDontReuseNames)

    def __init__(self):
        self._suffix = BTrees.Length.Length(0)

    def __iadd__(self, value):
        self._suffix.change(value)
        return self

    def __unicode__(self):
        return unicode(self._suffix.value)


name_suffix = zope.annotation.factory(
    NameSuffix, key='icemac.namechooser.DontReuseNames.NameSuffix')


class DontReuseNames(zope.container.contained.NameChooser):

    zope.component.adapts(
        icemac.addressbook.namechooser.interfaces.IDontReuseNames)

    def chooseName(self, name, object):
        container = self.context

        # remove characters that checkName does not allow
        name = unicode(name.replace('/', '-').lstrip('+@'))

        if not name:
            name = unicode(object.__class__.__name__)

        dot = name.rfind('.')
        if dot >= 0:
            extension = name[dot:]
            name = name[:dot]
        else:
            extension = ''

        name_suffix = icemac.addressbook.namechooser.interfaces.INameSuffix(
            container)
        while True:
            name_suffix += 1
            n = name + u'-' + unicode(name_suffix) + extension
            if n not in container:
                break

        # Make sure the name is valid.  We may have started with something bad.
        self.checkName(n, object)

        return n



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

from icemac.addressbook.i18n import _
import datetime
import icemac.addressbook.browser.table
import icemac.addressbook.interfaces
import icemac.truncatetext
import z3c.form.term
import z3c.table.column
import zope.component
import zope.i18n
import zope.preference.interfaces
import zope.schema.interfaces


END_OF_DATE = datetime.date(datetime.MAXYEAR, 12, 31)
END_OF_DATETIME = datetime.datetime(datetime.MAXYEAR, 12, 31, 23, 59, 59)


class BaseColumn(z3c.table.column.Column):
    """Column which is able to get an object on an attribute of the item and
    adapt it to specified interface."""

    firstAttrName = None  # Name of the attribute to be selected first
    attrInterface = None  # Adapt to this interface
    attrName = None  # Get the attribute with this name as value
    defaultValue = u''  # Value for display when there is no value.

    def getObject(self, item):
        "Get the object for which has the attribute display."
        if self.firstAttrName:
            item = getattr(item, self.firstAttrName)
        if self.attrInterface:
            # Need to remove the security proxy here as otherwise the
            # user defined field data cannot be accessed (it is stored
            # in an annotation). This is no security hole, as the
            # value is only used for display.
            item = zope.security.proxy.getObject(item)
            item = self.attrInterface(item)
        return item

    def getRawValue(self, item):
        "Compute the value, which can be None."
        return getattr(self.getObject(item), self.attrName)

    def getValue(self, item):
        "Compute the value, mostly ready for display."
        obj = self.getRawValue(item)
        if obj is None:
            return self.defaultValue
        return obj

    renderCell = getValue


class LinkColumn(z3c.table.column.LinkColumn,
                 BaseColumn):
    """LinkColumn which fetches link content from item."""

    def getSortKey(self, item):
        # Sort by the value displayed to the user, but lowercase it,
        # as the user does not expect lower case names below the
        # 'normal' upper case ones.
        return self.getLinkContent(item).lower()

    def getLinkContent(self, item):
        # Get the content from item or return the default value when empty.
        return self.getValue(item)

    def renderCell(self, item):
        if self.getLinkContent(item):
            return super(LinkColumn, self).renderCell(item)
        return self.defaultValue


class URLColumn(LinkColumn):
    """LinkColumn where the link URL is the same as the link content."""

    getLinkURL = LinkColumn.getLinkContent


class DateTimeColumn(z3c.table.column.FormatterColumn,
                 BaseColumn):
    """Column which is sortable even with `None` values in it."""

    maxValue = END_OF_DATETIME

    def renderCell(self, item):
        value = self.getRawValue(item)
        if value:
            return self.getFormatter().format(value)
        return self.defaultValue

    def getSortKey(self, item):
        key = self.getRawValue(item)
        if key is None:
            # empty date fields should be sorted to the end of the list
            key = self.maxValue
        return key


class DateColumn(DateTimeColumn):
    "DateColumn which is able to sort even `None` values."

    formatterCategory = 'date'
    maxValue = END_OF_DATE


class TruncatedContentColumn(BaseColumn):
    """Column which truncates its content."""

    length = 20  # number of characters to display
    ellipsis = u'…'  # ellipsis sign

    def renderCell(self, item):
        return icemac.truncatetext.truncate(
            self.getRawValue(item), self.length, self.ellipsis)


class BoolColumn(BaseColumn):
    "AdaptedGetAttrColumn for displaying bool values."

    def renderCell(self, item):
        value = self.getRawValue(item)
        # We use the labels of z3c.form here so the displayed values
        # are the same as in the edit form.
        if value is True:
            label = z3c.form.term.BoolTerms.trueLabel
        elif value is False:
            label = z3c.form.term.BoolTerms.falseLabel
        else:
            return self.defaultValue
        return zope.i18n.translate(label, context=self.request)


class TranslatedTiteledColumn(BaseColumn):
    """Column which returns the translated ITitle of the value."""

    def renderCell(self, obj):
        "Get the title of the value."
        value = self.getRawValue(obj)
        translated = self.defaultValue
        if value:
            title = icemac.addressbook.interfaces.ITitle(value)
            translated = zope.i18n.translate(title, context=self.request)
        return translated


class EMailColumn(BaseColumn,
                  z3c.table.column.EMailColumn):
    "Column which renders the cell contents as mailto-link."

    renderCell = z3c.table.column.EMailColumn.renderCell


def getColumnClass(entity, field):
    """Get a column class to display the requested field of an entity."""
    if field.__name__ in ('first_name', 'last_name'):
        # Person's first name and last name should be links:
        return LinkColumn
    if field.__name__ == 'keywords':
        # Keywords need a special column as they are an iterable:
        return icemac.addressbook.browser.table.KeywordsColumn
    if (zope.schema.interfaces.IText.providedBy(field) and
        not zope.schema.interfaces.ITextLine.providedBy(field)):
        # The content of text areas (but not text lines, which extend from
        # text area) should get truncated.
        return TruncatedContentColumn
    if zope.schema.interfaces.IDate.providedBy(field):
        # Date fields need a special column, as None values are not
        # compareable to date values:
        return DateColumn
    if field.__name__ == 'country':
        # Country is an object, so the translated title should be displayed:
        return TranslatedTiteledColumn
    if field.__name__ == 'email':
        # The e-mail address should be a mailto-link:
        return EMailColumn
    if field.__name__ == 'url':
        return URLColumn
    if icemac.addressbook.interfaces.IField.providedBy(field):
        # User defined fields:
        if field.type == u'Bool':
            return BoolColumn
        if field.type == u'URI':
            return URLColumn
        if field.type == u'Text':
            return TruncatedContentColumn
        if field.type == u'Date':
            return DateColumn
        if field.type == u'Datetime':
            return DateTimeColumn
    return BaseColumn


def getAdditionalColumnArgs(entity, field):
    """Get additional arguments to a specific column.

    Return a dict to be used as keyword args."""
    kw = dict()
    if icemac.addressbook.interfaces.IField.providedBy(field):
        # User defined fields need to be adapted
        kw['attrInterface'] = icemac.addressbook.interfaces.IUserFieldStorage
    if entity.interface != icemac.addressbook.interfaces.IPerson:
        # Addresses need the name of the default address attribute
        kw['firstAttrName'] = entity.tagged_values.get('default_attrib')
    if (field.__name__ == 'url' or
        (icemac.addressbook.interfaces.IField.providedBy(field) and
         field.type == 'URI')):
        kw['linkTarget'] = '_blank'
    return kw


class PersonList(icemac.addressbook.browser.table.PageletTable):
    """List persons in address book."""

    no_rows_message = _(
        u'There are no persons entered yet, click on "Add person" to create '
        u'one.')

    @property
    def prefs(self):
        "User defined preferences for person list."
        return zope.component.getUtility(
            zope.preference.interfaces.IPreferenceGroup, name="personList")

    def __init__(self, *args, **kw):
        super(PersonList, self).__init__(*args, **kw)
        self.sortOrder = self.prefs.sort_direction
        self.startBatchingAt = self.prefs.batch_size
        self.batchSize = self.prefs.batch_size
        self._columns = self._set_up_columns_and_sort_on()

    def _set_up_columns_and_sort_on(self):
        "Creates the columns and compute the order-by column."
        prefs = self.prefs
        order_by = prefs.order_by
        columns = []
        index = 0  # current column index
        # Entity and field of the column which sould be used for order-by:
        try:
            order_by_entity, order_by_field = (
                icemac.addressbook.preferences.sources.untokenize(
                    order_by))
        except KeyError:
            # Field has been deleted, so we can't use it for sorting:
            order_by_entity, order_by_field = None, None

        # Create all columns and compute the order-by column:
        for column_name in self.prefs.columns:
            try:
                entity, field = (
                    icemac.addressbook.preferences.sources.untokenize(
                        column_name))
            except KeyError:
                # Column no longer exists
                continue
            columns.append(
                z3c.table.column.addColumn(
                    self, getColumnClass(entity, field), field.__name__,
                    attrName=field.__name__, header=field.title,
                    weight=index, **getAdditionalColumnArgs(entity, field)
                    ))
            if entity == order_by_entity and field == order_by_field:
                # Found the entity and field for order by, so set the
                # expected name on the sortOn variable:
                self.sortOn = '%s-%s-%s' % (self.prefix, field.__name__, index)
            # current column exists, so the index can be adjusted
            index += 1
        return columns

    def setUpColumns(self):
        "Return the previously computed columns."
        return self._columns

    @property
    def values(self):
        "The values are stored on the context."
        return self.context.values()



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

import z3c.menu.ready2go
import z3c.menu.ready2go.manager
import zope.viewlet.manager


class IMasterData(z3c.menu.ready2go.ISiteMenu):
    """Containing viewlets which provide links to edit master data."""


MasterDataManager = zope.viewlet.manager.ViewletManager(
    'master-data', IMasterData, bases=(
        z3c.menu.ready2go.manager.MenuManager,))



import zope.component
import z3c.flashmessage.interfaces


class PageTestGetMessages(object):
    """Test page to send messages using z3c.flashmessage."""

    def __call__(self, msg):
        zope.component.getUtility(
            z3c.flashmessage.interfaces.IMessageSource).send(msg)
        return 'Message %r sent.' % msg



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

import sys
sys.path[0:0] = ['src']


import ConfigParser
import icemac.addressbook.install
import os.path
import subprocess
import shutil


USER_INI = 'install.user.ini'


def run_process(text, *args):
    print '%s ...' % text
    res = subprocess.call(args)
    if res:
        sys.exit(res)

def bool_get(config, key):
    "Read value from config."
    value = config.get('migration', key)
    return value == 'yes'

def migrate():
    # Read the ini file the configurator just created to get the
    # migration options.
    config = ConfigParser.SafeConfigParser()
    config.read(USER_INI)

    if not bool_get(config, 'do_migration'):
        # no migration wanted
        return
    old_instance = config.get('migration', 'old_instance')
    if not (old_instance and
            os.path.exists(os.path.join(old_instance, USER_INI))):
        print 'ERROR: You did not provide a path to the old instance.'
        print '       So I can not migrate the old content.'
        sys.exit(-1)
    cwd = os.getcwd()
    try:
        os.chdir(old_instance)
        demon_path = os.path.join('bin', 'addressbook')
        if bool_get(config, 'stop_server'):
            run_process('Stopping old instance', demon_path, 'stop')
        run_process('Creating backup of old instance',
                    os.path.join('bin', 'backup'))
        print 'Copying backup to new instance ...'
        # copy backup dir
        backup_dir = os.path.join('var', 'backups')
        os.rmdir(os.path.join(cwd, backup_dir))
        shutil.copytree(os.path.join(old_instance, backup_dir),
                        os.path.join(cwd, backup_dir))
        # copy blobs
        blobs_dir = os.path.join('var', 'blobs')
        shutil.copytree(os.path.join(old_instance, blobs_dir),
                        os.path.join(cwd, blobs_dir))
    finally:
        os.chdir(cwd)

    run_process('Restoring backup into new instance',
                os.path.join('bin', 'restore'))
    if bool_get(config, 'start_server'):
        run_process('Starting new instance', demon_path, 'start')


if __name__ == '__main__':
    python = sys.executable
    if not icemac.addressbook.install.check_prerequisites():
        sys.exit(-1)
    conf_args = []
    if len(sys.argv) > 1:
        conf_args.append(os.path.join(sys.argv[1], USER_INI))
    icemac.addressbook.install.Configurator(*conf_args)()

    run_process('running %s bootstrap.py' % python, python, 'bootstrap.py')
    run_process('running bin/buildout', 'bin/buildout')
    migrate()

    print 'Installation complete.'




import icemac.addressbook
import icemac.addressbook.browser.interfaces
import pkg_resources
import zope.component
import zope.contentprovider.interfaces
import zope.interface


class About(object):
    """View class for the about screen."""

    def version(self):
        return pkg_resources.get_distribution('icemac.addressbook').version


class CopyrightContentProvider(object):
    """Content provider for the copyright string."""
    zope.interface.implements(zope.contentprovider.interfaces.IContentProvider)
    zope.component.adapts(
        zope.interface.Interface,
        icemac.addressbook.browser.interfaces.IAddressBookLayer,
        zope.interface.Interface)

    def __init__(self, *args):
        pass

    def update(self):
        pass

    def render(self):
        return icemac.addressbook.copyright



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

import unittest
from icemac.addressbook.browser.file.file import cleanup_filename
import icemac.addressbook.testing


class Test_cleanup_filename(unittest.TestCase):

    def test_None_filename(self):
        self.assertEqual('<no name>', cleanup_filename(None))

    def test_simple_filename(self):
        self.assertEqual('sample.txt', cleanup_filename('sample.txt'))

    def test_Unix_filename(self):
        self.assertEqual(
            'sample.txt', cleanup_filename('/home/user/sample.txt'))

    def test_Windows_filename(self):
        self.assertEqual(
            'sample.txt', cleanup_filename(r'c:\Users\me\sample.txt'))

    def test_UNC_filename(self):
        self.assertEqual(
            'sample.txt', cleanup_filename(r'\\server\mine\sample.txt'))


def test_suite():
    return icemac.addressbook.testing.UnittestSuite(Test_cleanup_filename)



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: tests.py 1183 2010-11-23 17:17:09Z icemac $

import icemac.addressbook.testing
import zope.testing.renormalizing


def test_suite():
    suite = icemac.addressbook.testing.FunctionalDocFileSuite(
        "browser/about/about.txt",
        "browser/addressbook/addressbook.txt",
        "browser/authentication/login.txt",
        "browser/entities/bugfix.txt",
        "browser/entities/delete_choice_value.txt",
        "browser/entities/entities.txt",
        "browser/entities/file.txt",
        "browser/entities/sortorder.txt",
        "browser/export/export.txt",
        "browser/export/translation.txt",
        "browser/export/userfields.txt",
        "browser/keyword/keyword.txt",
        "browser/masterdata/masterdata.txt",
        "browser/person/clone.txt",
        "browser/person/file.txt",
        "browser/person/person.txt",
        "browser/person/sortorder.txt",
        "browser/person/translation.txt",
        "browser/principals/principals.txt",
        "browser/rootfolder/rootfolder.txt",
        "browser/search/delete.txt",
        "browser/search/search.txt",
        )
    suite.addTest(
        icemac.addressbook.testing.FunctionalDocFileSuite(
            # Tests which must not run with the default <DATETIME> normalizer:
            "browser/metadata.txt",
            checker=zope.testing.renormalizing.RENormalizing([])
            ))
    return suite



# -*- coding: utf-8 -*-
# Copyright (c) 2010 Michael Howitz
# See also LICENSE.txt

import unittest


class TestMetadata(unittest.TestCase):

    def test_implements(self):
        import zope.interface.verify
        import icemac.addressbook.metadata.interfaces
        import icemac.addressbook.metadata.storage
        self.assert_(zope.interface.verify.verifyObject(
            icemac.addressbook.metadata.interfaces.IEditor,
            icemac.addressbook.metadata.storage.EditorMetadataStorage()))



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: sources.py 847 2010-02-06 19:39:50Z icemac $

from icemac.addressbook.i18n import MessageFactory as _
import icemac.addressbook.export.interfaces
import zc.sourcefactory.basic
import zc.sourcefactory.interfaces
import zope.component
import zope.interface
import zope.publisher.browser


def get_exporter_tuples():
    """Get (name, instance) tuples of the registered exporters."""
    return zope.component.getAdapters(
        ([], zope.publisher.browser.TestRequest()),
        icemac.addressbook.export.interfaces.IExporter)


@zope.component.adapter(icemac.addressbook.export.interfaces.IExporter)
@zope.interface.implementer(zc.sourcefactory.interfaces.IToken)
def fromExporter(value):
    """The token must be the name for which the adapter is registered, so the
    adapter can be looked up later using this token."""
    for name, exporter in get_exporter_tuples():
        if value == exporter:
            return name


class ExporterSource(zc.sourcefactory.basic.BasicSourceFactory):

    def getValues(self):
        for name, value in get_exporter_tuples():
            yield value

    def getTitle(self, value):
        return _(u'${title} (${desc})',
                 mapping=dict(title=value.title, desc=value.description))

exporter_source = ExporterSource()



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: keyword.py 1183 2010-11-23 17:17:09Z icemac $

from icemac.addressbook.i18n import MessageFactory as _
import gocept.reference.interfaces
import icemac.addressbook.browser.base
import icemac.addressbook.browser.metadata
import icemac.addressbook.browser.table
import icemac.addressbook.interfaces
import z3c.form.button
import z3c.table.column


class AddForm(icemac.addressbook.browser.base.BaseAddForm):

    label = _(u'Add new keyword')
    interface = icemac.addressbook.interfaces.IKeyword
    class_ = icemac.addressbook.keyword.Keyword
    next_url = 'parent'


def can_delete_keyword(form):
    """Button condition telling if the displayed keyword is deleteable."""
    return (
        icemac.addressbook.browser.base.can_access('@@delete.html')(form)
        and
        not gocept.reference.interfaces.IReferenceTarget(
            form.context).is_referenced()
        )


class EditForm(icemac.addressbook.browser.base.GroupEditForm):

    groups = (icemac.addressbook.browser.metadata.MetadataGroup,)
    label = _(u'Edit keyword')
    interface = icemac.addressbook.interfaces.IKeyword
    next_url = 'parent'

    def applyChanges(self, data):
        # GroupForm has its own applyChanges but we need the one from
        # BaseEditForm here as inside the goups no changes are made
        # but there is a subscriber which raises an error which is
        # handled by BaseEditForm.
        return icemac.addressbook.browser.base.BaseEditForm.applyChanges(
            self, data)

    @z3c.form.button.buttonAndHandler(_('Apply'), name='apply')
    def handleApply(self, action):
        # because we define a new action we have to duplicate the
        # existing action because otherwise we'll loose it.
        super(EditForm, self).handleApply(self, action)

    @z3c.form.button.buttonAndHandler(_('Cancel'), name='cancel')
    def handleCancel(self, action):
        self.status = self.noChangesMessage

    @z3c.form.button.buttonAndHandler(
        _(u'Delete'), name='delete', condition=can_delete_keyword)
    def handleDelete(self, action):
        self.redirect_to_next_url('object', '@@delete.html')


class DeleteForm(icemac.addressbook.browser.base.BaseDeleteForm):
    label = _(u'Do you really want to delete this keyword?')
    interface = icemac.addressbook.interfaces.IKeyword
    field_names = ('title', )


class Table(icemac.addressbook.browser.table.Table):
    """List keywords in address book."""

    no_rows_message = _(u'No keywords defined yet.')

    def setUpColumns(self):
        "Return the previously computed columns."
        return [z3c.table.column.addColumn(
            self, icemac.addressbook.browser.table.TitleLinkColumn, 'keyword',
            header=_(u'keyword')),
                ]

    @property
    def values(self):
        "The values are stored on the context."
        return self.context.get_keywords()



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt

import icemac.addressbook.keyword
import unittest


class TestKeywords(unittest.TestCase):

    def setUp(self):
        self.keywords = icemac.addressbook.keyword.KeywordContainer()

    def test_get_keywords_empty(self):
        self.assertEqual([], list(self.keywords.get_keywords()))

    def test_get_keywords_not_empty(self):
        self.keywords['1'] = icemac.addressbook.keyword.Keyword(u'qwe')
        self.keywords['2'] = icemac.addressbook.keyword.Keyword(u'asd')
        self.keywords['3'] = icemac.addressbook.keyword.Keyword(u'dfg')
        self.keywords['4'] = icemac.addressbook.keyword.Keyword(u'bgr')
        self.assertEqual(['asd', 'bgr', 'dfg', 'qwe'],
                         sorted(x.title for x in self.keywords.get_keywords()))

    def test_get_keyword_by_title_found(self):
        self.keywords['1'] = icemac.addressbook.keyword.Keyword(u'foo')
        self.keywords['2'] = icemac.addressbook.keyword.Keyword(u'bar')
        self.assertEqual(self.keywords['1'],
                         self.keywords.get_keyword_by_title(u'foo'))
        self.assertEqual(self.keywords['2'],
                         self.keywords.get_keyword_by_title(u'bar'))

    def test_get_keyword_by_title_empty_container(self):
        self.assertEqual(None, self.keywords.get_keyword_by_title(u'asdf'))

    def test_get_keyword_by_title_not_found(self):
        self.keywords['1'] = icemac.addressbook.keyword.Keyword(u'asdf')
        self.assertEqual(None, self.keywords.get_keyword_by_title(u'foo'))
        self.assertEqual(None, self.keywords.get_keyword_by_title(u'bar'))

    def test_get_keyword_by_title_not_found_default(self):
        self.keywords['1'] = icemac.addressbook.keyword.Keyword(u'asdf')
        self.assertEqual('baz',
                         self.keywords.get_keyword_by_title(u'foo', 'baz'))



# -*- coding: latin-1 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt
# $Id: evolve5.py 783 2010-01-09 14:28:06Z icemac $

__docformat__ = "reStructuredText"


generation = 5


def evolve(context):
    """Failed approach to install the importer."""
    pass



# -*- coding: utf-8 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: simple.py 783 2010-01-09 14:28:06Z icemac $

import zope.session.interfaces

import icemac.addressbook.interfaces


class SimpleExport(object):

    def __call__(self):
        session = zope.session.interfaces.ISession(self.request)[
            icemac.addressbook.interfaces.PACKAGE_ID]
        persons = [self.context[id] for id in session['person_ids']]
        exporter = zope.component.getMultiAdapter(
            (persons, self.request),
            icemac.addressbook.export.interfaces.IExporter,
            name=session['exporter_token'])

        self.request.response.setHeader('Content-Type', exporter.mime_type)
        self.request.response.setHeader(
            'Content-Disposition',
            'attachment; filename=addressbook_export.%s' % (
                exporter.file_extension))

        return exporter.export()



# -*- coding: utf-8 -*-
# Copyright (c) 2010 Michael Howitz
# See also LICENSE.txt

import icemac.addressbook.metadata.interfaces
import zope.component
import zope.lifecycleevent
import zope.security.management
import zope.security.proxy


def set_current_princial_id(object, attribute):
    "Get the id of the current principal."
    interaction = zope.security.management.queryInteraction()
    if interaction is None:
        return
    for participation in interaction.participations:
        if participation.principal is not None:
            # Need to unwrap the object otherwise we get:
            # ForbiddenAttribute: ('__getitem__',
            #                      <BTrees.OOBTree.OOBTree object...>)
            # when trying to access the annotation key.
            unsafe_object = zope.security.proxy.getObject(object)
            metadata = icemac.addressbook.metadata.interfaces.IEditor(
                unsafe_object)
            setattr(
                metadata, attribute, unicode(participation.principal.title))
            # Seting the first participating principal is enough for now.
            return


@zope.component.adapter(
    zope.annotation.interfaces.IAttributeAnnotatable,
    zope.lifecycleevent.IObjectCreatedEvent)
def CreatorAnnotator(object, event):
    "Set creator on object."
    set_current_princial_id(object, 'creator')


def LastModifierAnnotator(object, event):
    "Set last modifier on object."
    set_current_princial_id(object, 'modifier')



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt
import icemac.addressbook.interfaces
import persistent
import persistent.interfaces
import zc.sourcefactory.basic
import zope.container.contained
import zope.dottedname.resolve
import zope.interface.interfaces
import zope.schema


MAIN_ENTITIES_NAME_SUFFIXES = [
    'person.Person',
    'address.PostalAddress',
    'address.PhoneNumber',
    'address.EMailAddress',
    'address.HomePageAddress',
    ]


def sorted_entities(entities):
    """Returns the entities sorted as defined in IEntityOrder."""
    order = zope.component.getUtility(
        icemac.addressbook.interfaces.IEntityOrder)
    order = [x for x in order]
    return sorted(entities, key=lambda x: order.index(x.name))


class Entities(object):
    "Predefined entities in the address book universe."

    zope.interface.implements(icemac.addressbook.interfaces.IEntities)

    def getEntities(self, sorted=True):
        """Get an iterable of all known entities."""
        entities = zope.component.getAllUtilitiesRegisteredFor(
            icemac.addressbook.interfaces.IEntity)
        if sorted:
            entities = sorted_entities(entities)
        return entities

    def getMainEntities(self, sorted=True):
        "Get an iterable of the most important entities."
        entities = [zope.component.getUtility(
                        icemac.addressbook.interfaces.IEntity,
                        name='icemac.addressbook.' + suffix)
                    for suffix in MAIN_ENTITIES_NAME_SUFFIXES]
        if sorted:
            entities = sorted_entities(entities)
        return entities


class PersistentEntities(Entities, zope.container.btree.BTreeContainer):
    "Predefined entities and user defined fields in the address book."


@zope.component.adapter(str)
@zope.interface.implementer(icemac.addressbook.interfaces.IEntity)
def entity_by_name(name):
    "Adapt Entity.name (not Entity.class_name!) to entity."
    entities = zope.component.getUtility(
        icemac.addressbook.interfaces.IEntities).getEntities(sorted=False)
    for candidate in entities:
        if candidate.name == name:
            return candidate
    raise ValueError("Unknown name: %r" % name)


@zope.component.adapter(zope.interface.interfaces.IInterface)
@zope.interface.implementer(icemac.addressbook.interfaces.IEntity)
def entity_by_interface(interface):
    "Adapt an interface to its entity."
    entities = zope.component.getUtility(
        icemac.addressbook.interfaces.IEntities).getEntities(sorted=False)
    for entity in entities:
        if entity.interface == interface:
            return entity
    # no entity found, create one on the fly, so all entities (even not
    # preconfigured ones) can be used the same way.
    return Entity(None, interface, None)


@zope.component.adapter(persistent.interfaces.IPersistent)
@zope.interface.implementer(icemac.addressbook.interfaces.IEntity)
def entity_by_obj(obj):
    "Adapt instance to entity."
    entities = zope.component.getUtility(
        icemac.addressbook.interfaces.IEntities).getEntities(sorted=False)
    for candidate in entities:
        if candidate.interface.providedBy(obj):
            return candidate
    raise ValueError("Unknown obj: %r" % obj)


class EntityOrder(object):
    """Global entity order utility."""

    zope.interface.implements(icemac.addressbook.interfaces.IEntityOrder)

    @property
    def order_storage(self):
        return zope.component.getUtility(
            icemac.addressbook.interfaces.IOrderStorage)

    def _get_entity_name(self, entity):
        try:
            return entity.name
        except ValueError:
            raise KeyError('Entity %r in not known in entity order.' % entity)

    def get(self, entity):
        """Get the index of the entity in the entity order."""
        return self.order_storage.get(
            self._get_entity_name(entity),
            icemac.addressbook.interfaces.ENTITIES)

    def isFirst(self, entity):
        """Tell whether `entity` comes first in the entity order."""
        return self.order_storage.isFirst(
            self._get_entity_name(entity),
            icemac.addressbook.interfaces.ENTITIES)

    def isLast(self, entity):
        """Tell whether `entity` comes last in the entity order."""
        return self.order_storage.isLast(
            self._get_entity_name(entity),
            icemac.addressbook.interfaces.ENTITIES)

    def __iter__(self):
        """Iterate over the entities sorted by order."""
        for entity in self.order_storage.__iter__(
            icemac.addressbook.interfaces.ENTITIES):
            yield entity

    def up(self, entity, delta=1):
        """Move the entity one position up in the entity order."""
        return self.order_storage.up(
            self._get_entity_name(entity),
            icemac.addressbook.interfaces.ENTITIES, delta)

    def down(self, entity, delta=1):
        """Move the entity one position down in the entity order."""
        return self.order_storage.down(
            self._get_entity_name(entity),
            icemac.addressbook.interfaces.ENTITIES, delta)


class ChoiceFieldValuesSource(zc.sourcefactory.basic.BasicSourceFactory):
    "Source containing the values of a choice field."

    def __init__(self, values):
        super(ChoiceFieldValuesSource, self).__init__()
        self.values = values

    def getValues(self):
        return self.values

    def getTitle(self, value):
        return value


def user_field_to_schema_field(field):
    """Convert a user defined field (IField) into a zope.schema field."""
    if field.type == 'Choice':
        schema_field = zope.schema.Choice(
            title=field.title, required=False,
            source=ChoiceFieldValuesSource(field.values))
    else:
        schema_field = getattr(zope.schema, field.type)(
            title=field.title, required=False)
    schema_field.__name__ = str(field.__name__)
    schema_field.interface = (
        icemac.addressbook.interfaces.IUserFieldStorage)
    return schema_field


class FakeObject(object):
    "We need an instance to provide an interface for the `getAdapters` call."


class Entity(object):
    "An entity int the address book universe."
    # Use `create_entity` factory for easier usage.

    zope.interface.implements(icemac.addressbook.interfaces.IEntity)

    title = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IEntity['title'])
    interface = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IEntity['interface'])
    class_name = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IEntity['class_name'])

    def __init__(self, title, interface, class_name, **kw):
        # To create an entity, use `create_entity` factory (see below) which
        # also does the ZCA set up.
        self.title = title
        self.interface = interface
        self.class_name = class_name
        # getRawFields needs an instance which provides the interface to
        # look up the user defined fields which are named adapters
        self._fake_object = FakeObject()
        zope.interface.directlyProvides(self._fake_object, self.interface)
        # Additional keyword arguments are stored as tagged values
        self._tagged_values = kw

    @property
    def name(self):
        "Uniqe name of the entity which only contains letters."
        if not self.class_name:
            raise ValueError(
                "Entity has no `class_name` set, so `name` cannot be computed."
                )
        parts = self.class_name.replace('_', '.').split('.')
        return ''.join(x.capitalize() for x in parts)

    @property
    def tagged_values(self):
        "Dict of tagged values of the entity."
        return self._tagged_values.copy()

    def getRawFields(self):
        """Get ordered name, field tuples of the schema fields on the entity.

        Returnes static (zope.schema) and user defined (IField) fields.

        """
        for name, field in zope.schema.getFieldsInOrder(self.interface):
            yield name, field
        # self._fake_object is needed here as the interfaces provided by the
        # objects are used in the look up
        adapters = zope.component.getAdapters(
            (self, self._fake_object), icemac.addressbook.interfaces.IField)
        for name, field in adapters:
            yield str(field.__name__), field

    def getFieldsInOrder(self):
        """Get ordered name, field tuples of the schema fields on the entity.

        Converts user defined fields (see IField) into zope.schema fields.

        """
        for name, field in self.getRawFields():
            if icemac.addressbook.interfaces.IField.providedBy(field):
                yield name, user_field_to_schema_field(field)
            else:
                yield name, field

    def getFieldValuesInOrder(self):
        """Get ordered list of the schema fields on the entity.

        Converts user defined fields (see IField) into zope.schema fields.

        """
        return [field for name, field in self.getFieldsInOrder()]

    def getRawField(self, field_name):
        """Get a field by its name."""
        return dict(self.getRawFields())[field_name]

    def getField(self, field_name):
        """Get a zope.schema field by its name."""
        return dict(self.getFieldsInOrder())[field_name]

    def getClass(self):
        """Get the class object for `self.class_name`."""
        if self.class_name:
            return zope.dottedname.resolve.resolve(self.class_name)
        raise ValueError("class_name is not set.")


def create_entity(title, interface, class_, **kw):
    "Factory to create an entity and to the ZCA set up."
    class_name = '%s.%s' % (class_.__module__, class_.__name__)
    entity = Entity(title, interface, class_name, **kw)
    zope.interface.classImplements(
        class_, icemac.addressbook.interfaces.IMayHaveUserFields)
    return entity


class Field(persistent.Persistent, zope.container.contained.Contained):
    """User defined field."""

    zope.interface.implements(icemac.addressbook.interfaces.IField)

    type = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IField['type'])
    title = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IField['title'])
    values = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IField['values'])
    notes = zope.schema.fieldproperty.FieldProperty(
        icemac.addressbook.interfaces.IField['notes'])


class FieldAdapterFactory(persistent.Persistent):
    "Factory to register a field as an adapter."

    def __init__(self, field):
        self._field = field

    def __call__(self, *args):
        return self._field


def store_and_register_field(field, entity):
    "Store a `field` in the ZODB and register it for an `entity`."
    # store in entities utility
    parent = zope.component.getUtility(icemac.addressbook.interfaces.IEntities)
    name = icemac.addressbook.utils.add(parent, field)
    # register as adapter
    sm = zope.site.hooks.getSiteManager()
    sm.registerAdapter(
        FieldAdapterFactory(field),
        provided=icemac.addressbook.interfaces.IField,
        required=(icemac.addressbook.interfaces.IEntity, entity.interface),
        name=name)
    return name


class FieldStorage(persistent.Persistent):
    """Storage for field values in annotations."""
    zope.component.adapts(
        icemac.addressbook.interfaces.IMayHaveUserFields)
    zope.interface.implements(
        icemac.addressbook.interfaces.IUserFieldStorage)

    def __getattr__(self, attrib):
        # We have no default values on the class nor we might know them.
        if not attrib.startswith('__'):
            return self.__dict__.get(attrib, None)
        # Cloning a person tries to access __reduce_ex__ and
        # __reduce__ which are not in __dict__. It is expeced that
        # they are either methods or not existing, so None is no valid
        # return value.
        raise AttributeError(attrib)


field_storage = zope.annotation.factory(
    FieldStorage, key='icemac.userfield.storage')



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: evolve7.py 1183 2010-11-23 17:17:09Z icemac $

__docformat__ = "reStructuredText"


import icemac.addressbook.generations.utils


generation = 7


def evolve(context):
    """Install user defined fields utility.
    """
    icemac.addressbook.generations.utils.update_address_book_infrastructure(
        context)



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

import doctest
import icemac.addressbook.namechooser.namechooser
import zope.annotation.attribute
import zope.testing.cleanup
import zope.testrunner.layer


class NameChooserLayer(zope.testrunner.layer.UnitTests):
    """Layer for name chooser tests."""

    @classmethod
    def setUp(cls):
        zope.component.provideAdapter(
            zope.annotation.attribute.AttributeAnnotations)
        zope.component.provideAdapter(
            icemac.addressbook.namechooser.namechooser.name_suffix)

    @classmethod
    def tearDown(cls):
        zope.testing.cleanup.tearDown()


def test_suite():
    suite = doctest.DocFileSuite('namechooser.txt')
    suite.layer = NameChooserLayer
    return suite



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: sources.py 1183 2010-11-23 17:17:09Z icemac $

from icemac.addressbook.i18n import MessageFactory as _
import stabledict
import zc.sourcefactory.basic
import zc.sourcefactory.contextual
import zope.component


class TitleMappingSource(zc.sourcefactory.basic.BasicSourceFactory):
    "Abstract base class for sources using a mapping between value and title."

    _mapping = None  # to be set in child class

    def getValues(self):
        return self._mapping.keys()

    def getTitle(self, value):
        return self._mapping[value]


class YesNoSource(TitleMappingSource):
    _mapping = stabledict.StableDict(
        ((True, _(u'yes')),
         (False, _(u'no'))))

yes_no_source = YesNoSource()


class FieldTypeSource(TitleMappingSource):
    _mapping = stabledict.StableDict(
        ((u'Bool', _(u'bool')),
         (u'Choice', _('choice')),
         (u'Date', _(u'date')),
         (u'Datetime', _(u'datetime')),
         (u'Decimal', _('decimal number')),
         (u'Int', _(u'integer number')),
         (u'Text', _(u'text area')),
         (u'TextLine', _(u'text line')),
         (u'URI', _(u'URL')),
         ))


class KeywordSource(zc.sourcefactory.basic.BasicSourceFactory):
    "Source of keywords in the address book."

    def getValues(self):
        import icemac.addressbook.interfaces  # avoid circular import
        keywords = zope.component.getUtility(
            icemac.addressbook.interfaces.IKeywords)
        return sorted(keywords.get_keywords(), key=lambda x: x.title.lower())

    def getTitle(self, value):
        return value.title

keyword_source = KeywordSource()


class ContextByInterfaceSource(
    zc.sourcefactory.contextual.BasicContextualSourceFactory):
    "Source to select objects from context container by a given interface."

    def __init__(self, interface):
        super(ContextByInterfaceSource, self).__init__()
        self.interface = interface

    def getValues(self, context):
        for value in context.values():
            if self.interface.providedBy(value):
                yield value

    def getTitle(self, context, value):
        import icemac.addressbook.interfaces  # avoid circular import
        return icemac.addressbook.interfaces.ITitle(value)



# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: __init__.py 1152 2010-10-28 18:48:52Z icemac $
"""Database initialisation and upgrading."""

import zope.generations.generations


GENERATION = 16


manager = zope.generations.generations.SchemaManager(
    minimum_generation=GENERATION,
    generation=GENERATION,
    package_name='icemac.addressbook.generations')



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt
# $Id: testing.py 1183 2010-11-23 17:17:09Z icemac $

import icemac.addressbook.address
import icemac.addressbook.addressbook
import icemac.addressbook.file.file
import icemac.addressbook.interfaces
import icemac.addressbook.keyword
import icemac.addressbook.person
import icemac.addressbook.principals.principals
import icemac.addressbook.principals.sources
import icemac.addressbook.utils
import inspect
import os
import os.path
import re
import tempfile
import unittest
import z3c.etestbrowser.wsgi
import zope.annotation.attribute
import zope.app.wsgi.testlayer
import zope.component
import zope.site.hooks
import zope.testbrowser.browser
import zope.testbrowser.interfaces
import zope.testing.cleanup
import zope.testing.renormalizing
import zope.testrunner.layer

if os.environ.get('ZOPETESTINGDOCTEST'):
    from zope.testing import doctest
else:
    import doctest


class AddressBookUnitTests(zope.testrunner.layer.UnitTests):
    """Layer for gathering addressbook unit tests."""

    @classmethod
    def setUp(cls):
        # Some classes use gocept.reference and store default values for
        # which annotations are needed.
        zope.component.provideAdapter(
            zope.annotation.attribute.AttributeAnnotations)

    @classmethod
    def tearDown(cls):
        zope.testing.cleanup.tearDown()


def UnittestSuite(*classes):
    """Create a unittest test suite from unittest test classes."""
    suite = unittest.TestSuite()
    for class_ in classes:
        suite.addTest(unittest.makeSuite(class_))
    return suite


def AddressBookUnittestSuite(*classes):
    """Create a unittest test suite on addressbook layer."""
    suite = UnittestSuite(*classes)
    suite.layer = AddressBookUnitTests
    return suite


FunctionalLayer = zope.app.wsgi.testlayer.BrowserLayer(icemac.addressbook)


class FunctionalTestCase(unittest.TestCase):
    """Base class for functional tests."""

    layer = FunctionalLayer


class AddressBookFunctionalTestCase(FunctionalTestCase):
    "Functional test case where the address book gets created in the set up."

    def setUp(self):
        super(AddressBookFunctionalTestCase, self).setUp()
        self.old_site = zope.site.hooks.getSite()
        zope.site.hooks.setSite(
            icemac.addressbook.testing.create_addressbook(
                parent=self.layer.getRootFolder()))

    def tearDown(self):
        super(AddressBookFunctionalTestCase, self).tearDown()
        zope.site.hooks.setSite(self.old_site)


def FunctionalDocFileSuite(*paths, **kw):
    kw['optionflags'] = (kw.get('optionflags', 0) |
                         doctest.ELLIPSIS |
                         doctest.NORMALIZE_WHITESPACE)
    if 'layer' in kw:
        layer = kw.pop('layer')
    else:
        layer = FunctionalLayer
    globs = kw.setdefault('globs', {})
    globs['getRootFolder'] = layer.getRootFolder
    if 'checker' not in kw:
        kw['checker'] = zope.testing.renormalizing.RENormalizing([
            (re.compile(r'[0-9]{2}/[0-9]{2}/[0-9]{2} [0-9]{2}:[0-9]{2}'),
             '<DATETIME>')
            ])
    suite = doctest.DocFileSuite(*paths, **kw)
    suite.layer = layer
    return suite


def _get_control_names(interface, browser=None, form=None):
    """Get a sorted list of names of controls providing the given interface.

    Whether the browser was given as argument the form of the browser is used.
    """
    if browser is not None:
        form = browser.getForm()
    else:
        browser = form.browser
    names = []
    for ctrl in form.mech_form.controls:
        control = zope.testbrowser.browser.controlFactory(ctrl, form, browser)
        if interface.providedBy(control):
            names.append(control.name)
    return sorted(names)


def get_submit_control_names(browser=None, form=None):
    """Get a list of the names of the submit controls in the form.

    Whether the browser was given as argument the form of the browser is used.
    """
    return _get_control_names(
        zope.testbrowser.interfaces.ISubmitControl, browser, form)


def get_all_control_names(browser=None, form=None):
    """Get a list of all names of the controls in the form.

    Whether the browser was given as argument the form of the browser is used.
    """
    return _get_control_names(
        zope.testbrowser.interfaces.IControl, browser, form)


def get_messages(browser):
    """Return the info messages displayed in browser.

    Returns string when there is exactly one message, list otherwise."""
    if not isinstance(browser, z3c.etestbrowser.wsgi.ExtendedTestBrowser):
        raise ValueError(
            'browser must be z3c.etestbrowser.wsgi.ExtendedTestBrowser')
    return [x.text
            for x in browser.etree.xpath('//div[@id="info-messages"]/ul/li')]


def in_out_widget_select(browser, control_name, select_controls):
    """Function to add a selection to an in-out-widget.

    browser ... testbrowser instance
    control_name ... name of the in-out-control, something like
                     form.widgets.columns
    select_controls ... list of control instances (item controls of in or out
                        list) which should be selected
    """
    form = browser.getForm()
    for control in select_controls:
        form.mech_form.new_control(
            type='hidden',
            name='%s:list' % control_name,
            attrs=dict(value=control.optionValue))


def write_temp_file(content, suffix):
    "Write `content` to a temporary file and return file handle and file name."
    fd, filename = tempfile.mkstemp(suffix=suffix)
    os.write(fd, content)
    os.close(fd)
    return file(filename, 'r'), os.path.basename(filename)


### Helper functions to create objects in the database ###


def create_addressbook(name='ab', title=u'test address book', parent=None):
    """Create an address book.

    When parent is `None`, it gets created in the root folder of the data base.

    """
    ab = icemac.addressbook.utils.create_obj(
        icemac.addressbook.addressbook.AddressBook, title=title)
    frame = inspect.currentframe()
    if parent is None:
        try:
            parent = frame.f_back.f_globals['getRootFolder']()
        finally:
            del frame
    parent[name] = ab
    return ab


def create_keyword(addressbook, title, return_obj=True):
    parent = addressbook.keywords
    name = icemac.addressbook.utils.create_and_add(
        parent, icemac.addressbook.keyword.Keyword, title=title)
    if return_obj:
        return parent[name]


@icemac.addressbook.utils.set_site
def create_person(parent, last_name, return_obj=True, **kw):
    """Create a new person in the address book.

    Caution: The created person is not editable trough the UI, as it
    is missing the default entries. See also `create_full_person`.
    """
    kw['last_name'] = last_name
    name = icemac.addressbook.utils.create_and_add(
        parent, icemac.addressbook.person.Person, **kw)
    if return_obj:
        return parent[name]


@icemac.addressbook.utils.set_site
def create_full_person(parent, last_name, return_obj=True, **kw):
    """Create a new person in the address book with all default entries.

    The created person is editable through the UI.
    """
    person = create_person(parent, parent, last_name, return_obj=True, **kw)
    create_postal_address(parent, person)
    create_email_address(parent, person)
    create_home_page_address(parent, person)
    create_phone_number(parent, person)
    if return_obj:
        return person


@icemac.addressbook.utils.set_site
def create_postal_address(person, set_as_default=True, return_obj=True, **kw):
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.address.PostalAddress, **kw)
    address = person[name]
    if set_as_default:
        person.default_postal_address = address
    if return_obj:
        return address


@icemac.addressbook.utils.set_site
def create_email_address(person, set_as_default=True, return_obj=True, **kw):
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.address.EMailAddress, **kw)
    address = person[name]
    if set_as_default:
        person.default_email_address = address
    if return_obj:
        return address


@icemac.addressbook.utils.set_site
def create_home_page_address(
    person, set_as_default=True, return_obj=True, **kw):
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.address.HomePageAddress, **kw)
    address = person[name]
    if set_as_default:
        person.default_home_page_address = address
    if return_obj:
        return address


@icemac.addressbook.utils.set_site
def create_phone_number(person, set_as_default=True, return_obj=True, **kw):
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.address.PhoneNumber, **kw)
    number = person[name]
    if set_as_default:
        person.default_phone_number = number
    if return_obj:
        return number


@icemac.addressbook.utils.set_site
def create_file(person, return_obj=True, **kw):
    "Create a file object inside the `person`."
    name = icemac.addressbook.utils.create_and_add(
        person, icemac.addressbook.file.file.File, **kw)
    if return_obj:
        return person[name]


@icemac.addressbook.utils.set_site
def create(
    parent, entity_name, return_obj=False, set_as_default=False, *args, **kw):
    """Create an object using an entity.

    entity_name ... module name and class name of entity

    """
    entity = zope.component.getUtility(
        icemac.addressbook.interfaces.IEntity, name=entity_name)
    name = icemac.addressbook.utils.create_and_add(
        parent, entity.getClass(), *args)
    obj = parent[name]
    for key, value in kw.items():
        field = entity.getField(key)
        context = field.interface(obj)
        field.set(context, value)
    if set_as_default:
        field = icemac.addressbook.person.get_default_field(entity.interface)
        field.set(parent, obj)
    if return_obj:
        return obj


@icemac.addressbook.utils.set_site
def create_user(ab, first_name, last_name, email, password, roles, **kw):
    person = create_person(
        ab, ab, last_name, first_name=first_name, **kw)
    create_email_address(ab, person, email=email)

    role_factory = icemac.addressbook.principals.sources.role_source.factory
    role_values = role_factory.getValues()
    selected_roles = []
    for role_title in roles:
        for candidate in role_values:
            if role_factory.getTitle(candidate) == role_title:
                selected_roles.append(candidate)
                break
    icemac.addressbook.utils.create_and_add(
        ab.principals, icemac.addressbook.principals.principals.Principal,
        person=person, password=password, roles=selected_roles)


@icemac.addressbook.utils.set_site
def create_field(entity_name, type, title, **kw):
    """Create a user defined field for an entity.

    entity_name ... module name and class name of entity
    type ... see values of sources.FieldTypeSource

    """
    field = icemac.addressbook.utils.create_obj(
        icemac.addressbook.entities.Field, type=type, title=title, **kw)
    entity = zope.component.getUtility(
        icemac.addressbook.interfaces.IEntity, name=entity_name)
    icemac.addressbook.entities.store_and_register_field(field, entity)



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: tests.py 1183 2010-11-23 17:17:09Z icemac $

import icemac.addressbook.export.base
import icemac.addressbook.export.interfaces
import icemac.addressbook.export.xls.simple
import icemac.addressbook.testing
import unittest
import zope.interface.verify


class TestInterfaces(unittest.TestCase):

    def test_BaseExport(self):
        zope.interface.verify.verifyObject(
            icemac.addressbook.export.interfaces.IExporter,
            icemac.addressbook.export.base.BaseExporter([], None))

    def test_XLSExport(self):
        zope.interface.verify.verifyObject(
            icemac.addressbook.export.interfaces.IExporter,
            icemac.addressbook.export.xls.simple.XLSExport([], None))

    def test_XLSDefaultsExport(self):
        zope.interface.verify.verifyObject(
            icemac.addressbook.export.interfaces.IExporter,
            icemac.addressbook.export.xls.simple.DefaultsExport([], None))

    def test_XLSCompleteExport(self):
        zope.interface.verify.verifyObject(
            icemac.addressbook.export.interfaces.IExporter,
            icemac.addressbook.export.xls.simple.CompleteExport([], None))


class Test__eq__(unittest.TestCase):

    def setUp(self):
        self.exporter = icemac.addressbook.export.base.BaseExporter(
            [], None)

    def test_not__eq__other_type_number(self):
        self.assertEqual(False, self.exporter == 1)

    def test_not__eq__other_type_object(self):
        self.assertEqual(False, self.exporter == object())

    def test_not__eq__other_class(self):
        other_exporter = icemac.addressbook.export.xls.simple.XLSExport(
            [], None)
        self.assertEqual(False, self.exporter == other_exporter)

    def test___eq__other_instance_same_class(self):
        other_exporter = icemac.addressbook.export.base.BaseExporter(
            [], None)
        self.assertEqual(True, self.exporter == other_exporter)


class Test_export(unittest.TestCase):
    """Test export method of BaseExporter to get better test coverage."""

    def test_export(self):
        exporter = icemac.addressbook.export.base.BaseExporter(None)
        self.assertRaises(NotImplementedError, exporter.export)


def test_suite():
    suite = icemac.addressbook.testing.UnittestSuite(
        TestInterfaces, Test__eq__, Test_export)
    suite.addTest(icemac.addressbook.testing.FunctionalDocFileSuite(
        'export/export.txt',
        'export/sortorder.txt',
        'export/userfields.txt',
        ))
    return suite



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: i18n.py 1009 2010-08-15 14:20:55Z icemac $

"""I18n message factory for icemac.addressbook. """

import zope.i18nmessageid

_ = MessageFactory = zope.i18nmessageid.MessageFactory('icemac.addressbook')



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

"""Some entity stubs."""

import icemac.addressbook.entities
import icemac.addressbook.interfaces
import persistent
import zope.component
import zope.interface


class IDuck(zope.interface.Interface):
    pass


class Duck(object):
    zope.interface.implements(IDuck)


class ICat(zope.interface.Interface):
    pass


class Cat(object):
    zope.interface.implements(ICat)


class IKwack(zope.interface.Interface):
    pass


class Kwack(persistent.Persistent):
    zope.interface.implements(IKwack)


class IDog(zope.interface.Interface):
    pass


def setUpStubEntities(self, entities_class):
    """Create and register the stub entities."""
    self.entities = entities_class()
    zope.component.provideUtility(
        self.entities, icemac.addressbook.interfaces.IEntities)
    # entities
    self.cat = icemac.addressbook.entities.create_entity(
        u'Cat', ICat, Cat, default_attrib='default_cat')
    zope.component.provideUtility(self.cat, name=self.cat.class_name)
    self.duck = icemac.addressbook.entities.create_entity(
        u'Duck', IDuck, Duck, default_attrib='default_duck')
    zope.component.provideUtility(self.duck, name=self.duck.class_name)
    self.kwack = icemac.addressbook.entities.create_entity(
        u'Kwack', IKwack, Kwack, default_attrib='default_kwack')
    zope.component.provideUtility(self.kwack, name=self.kwack.class_name)



import unittest


class MockEntitiesAndOrder(object):
    """Mock wich implements parts of IEntities and IEntityOrder."""

    def __init__(self):
        import icemac.addressbook.address
        self.order = {
            icemac.addressbook.address.postal_address_entity: 0,
            icemac.addressbook.address.phone_number_entity: 10,
            icemac.addressbook.address.e_mail_address_entity: 20,
            icemac.addressbook.address.home_page_address_entity: 30,
            }

    def get(self, entity):
        return self.order[entity]

    def getEntities(self, **kw):
        return self.order.keys()


class PersonDefaultsEntityTest(unittest.TestCase):

    def setUp(self):
        import icemac.addressbook.interfaces
        import zope.component.testing

        zope.component.testing.setUp()
        mock = MockEntitiesAndOrder()
        zope.component.provideUtility(
            mock, icemac.addressbook.interfaces.IEntityOrder)
        zope.component.provideUtility(
            mock, icemac.addressbook.interfaces.IEntities)

    def tearDown(self):
        import zope.component.testing
        zope.component.testing.tearDown()

    def callFUT(self):
        import icemac.addressbook.person
        pde = icemac.addressbook.person.PersonDefaultsEntity(
            None, icemac.addressbook.interfaces.IPersonDefaults, None)
        return [x[0] for x in pde.getRawFields()]

    def test_default_sortorder(self):
        self.assertEqual(
            ['default_postal_address', 'default_phone_number',
             'default_email_address', 'default_home_page_address'],
            self.callFUT())

    def test_changed_sortorder(self):
        import icemac.addressbook.address
        import icemac.addressbook.interfaces
        import zope.component

        # Switching first two entries:
        entity_order = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntityOrder)
        entity_order.order[
            icemac.addressbook.address.postal_address_entity] = 15
        self.assertEqual(
            ['default_phone_number', 'default_postal_address',
             'default_email_address', 'default_home_page_address'],
            self.callFUT())



# -*- coding: utf-8 -*-
# Copyright (c) 2010 Michael Howitz
# See also LICENSE.txt

# Copyright string which can be recognized by fix-copyright script:
copyright = '''
# Copyright (c) 2008-2010 Michael Howitz
'''[13:-1]



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt

from icemac.addressbook.i18n import MessageFactory as _
import classproperty
import gocept.reference.interfaces
import icemac.addressbook.browser.interfaces
import icemac.addressbook.browser.resource
import icemac.addressbook.interfaces
import icemac.addressbook.utils
import transaction
import z3c.flashmessage.interfaces
import z3c.form.button
import z3c.form.field
import z3c.form.group
import z3c.formui.form
import zope.component
import zope.interface
import zope.security
import zope.security.interfaces
import zope.security.proxy
import zope.site.hooks
import zope.traversing.api
import zope.traversing.browser.interfaces
import zope.traversing.publicationtraverse


class display_title(object):
    """ITitle adapter which can be used in PageTemplate using
    ``obj/@@display_title``.

    """
    zope.component.adapts(
        zope.interface.Interface,
        icemac.addressbook.browser.interfaces.IAddressBookLayer)
    zope.interface.implements(icemac.addressbook.interfaces.ITitle)

    def __init__(self, context, request):
        self.context = context

    def __call__(self):
        return icemac.addressbook.interfaces.ITitle(self.context)


def create(form, class_, data):
    obj = icemac.addressbook.utils.create_obj(class_)
    z3c.formui.form.applyChanges(form, obj, data)
    return obj


def all_(*constraints):
    """All given button constraints must evaluate to true."""
    return lambda form: all(constraint(form) for constraint in constraints)


def get_needed_resources(self):
    """Get the needed resources for self."""
    if self.need:
        getattr(icemac.addressbook.browser.resource, self.need).need()


class FlashView(object):
    """Base class to send flash messages."""

    def send_flash(self, message):
        zope.component.getUtility(
            z3c.flashmessage.interfaces.IMessageSource).send(message)


class BaseView(FlashView):
    "Base for view classes."

    need = None  # name of a resource from icemac.addressbook.browser.resource

    def update(self):
        get_needed_resources(self)
        super(BaseView, self).update()

    def url(self, obj):
        adapter = zope.component.getMultiAdapter(
            (obj, self.request),
            zope.traversing.browser.interfaces.IAbsoluteURL)
        return adapter()


class BaseForm(BaseView):
    """Base for all forms."""

    interface = None  # interface for form
    need = 'form_css'

    # privat
    _status = ''

    @property
    def fields(self):
        if self.interface is None:
            field_values = ()
        else:
            fields = icemac.addressbook.interfaces.IEntity(self.interface)
            field_values = fields.getFieldValuesInOrder()
        return z3c.form.field.Fields(*field_values)

    class status(classproperty.classproperty):
        def __get__(self):
            return self._status

        def __set__(self, message):
            if message != z3c.form.form.Form.formErrorsMessage:
                # Non-error messages must be handled by z3c.flashmessage,
                # too, as they must be displayed on the next page after the
                # redirect. But we need them to determine whether a redirect
                # is necessary, too.
                self.send_flash(message)
            self._status = message


class BaseAddForm(BaseForm, z3c.formui.form.AddForm):
    """Simple base add form."""

    class_ = None  # create object from this class
    next_url = None  # target after creation, one of ('object', 'parent')

    def createAndAdd(self, data):
        # overwriting method as otherwise object created event would
        # get send twice
        self.obj = self.create(data)
        self.add(self.obj)
        return self.obj

    def create(self, data):
        return create(self, self.class_, data)

    def add(self, obj):
        try:
            self._name = icemac.addressbook.utils.add(self.context, obj)
        except zope.interface.Invalid, e:
            transaction.doom()
            raise z3c.form.interfaces.ActionExecutionError(e)

    @z3c.form.button.buttonAndHandler(_('Add'), name='add')
    def handleAdd(self, action):
        # because we define a new action we have to duplicate the
        # existing action because otherwise we'll lose it.
        super(BaseAddForm, self).handleAdd(self, action)
        if self._finishedAdd:
            self.status = _(
                '"${title}" added.',
                mapping=dict(title=icemac.addressbook.interfaces.ITitle(
                    self.obj)))

    @z3c.form.button.buttonAndHandler(_('Cancel'), name='cancel')
    def handleCancel(self, action):
        self.next_url = 'parent'
        self._finishedAdd = True
        self.status = _('Addition canceled.')

    def nextURL(self):
        if self.next_url == 'object':
            context = self.context[self._name]
        elif self.next_url == 'parent':
            context = self.context
        else:
            raise ValueError("Don't know how to handle next_url %r.")
        return self.url(context)


def update_with_redirect(class_, self):
    "Call update of super class and redirect when necessary."
    get_needed_resources(self)
    super(class_, self).update()
    if self.request.response.getStatus() in (302, 303, 304):
        # already redirecting
        return
    if self.status in (self.successMessage, self.noChangesMessage):
        self.redirect_to_next_url()


class BaseEditForm(BaseForm, z3c.formui.form.EditForm):
    """Base Edit form."""

    next_url = None  # target object after edit, one of ('object', 'parent')
    next_view = None  # target view after edit (None for default view)
    id = 'edit-form'

    def update(self):
        update_with_redirect(BaseEditForm, self)

    def redirect_to_next_url(self, next_url=None, next_view=None):
        if next_url is None:
            next_url = self.next_url
        if next_url == 'object':
            target = self.context
        elif next_url == 'parent':
            target = zope.traversing.api.getParent(self.context)
        elif next_url == 'site':
            target = zope.site.hooks.getSite()
        else:
            raise ValueError('next_url %r unknown' % next_url)
        target_url = self.url(target)
        if next_view is None:
            next_view = self.next_view
        if next_view:
            target_url += '/%s' % next_view
        self.request.response.redirect(target_url)

    def applyChanges(self, data):
        try:
            return super(BaseEditForm, self).applyChanges(data)
        except zope.interface.Invalid, e:
            transaction.doom()
            raise z3c.form.interfaces.ActionExecutionError(e)


class BaseEditFormWithCancel(BaseEditForm):
    "BaseEditForm with cancel button."

    @z3c.form.button.buttonAndHandler(_('Apply'), name='apply')
    def handleApply(self, action):
        # because we define a new action we have to duplicate the
        # existing action because otherwise we'll lose it.
        super(BaseEditFormWithCancel, self).handleApply(self, action)

    @z3c.form.button.buttonAndHandler(_('Cancel'), name='cancel')
    def handleCancel(self, action):
        self.status = self.noChangesMessage


class GroupEditForm(z3c.form.group.GroupForm, BaseEditForm):
    "BaseEditForm as group form."

    # This is needed here as GroupForm does not do a super-call.
    def update(self):
        update_with_redirect(GroupEditForm, self)


class BaseDeleteForm(BaseEditForm):
    "Display a deletion confirmation dialog."

    label = _(u'Do you really want to delete this entry?')
    requiredInfo = None  # do never display requiredInfo
    interface = None
    field_names = ()  # tuple of field names for display; empty for all
    next_view_after_delete = None  # when None, use same view as self.next_view

    need = 'no_max_content_css'
    mode = z3c.form.interfaces.DISPLAY_MODE
    next_url = 'object'

    @property
    def fields(self):
        fields = super(BaseDeleteForm, self).fields
        if self.field_names:
            fields = fields.select(*self.field_names)
        return fields

    @z3c.form.button.buttonAndHandler(_(u'No, cancel'), name='cancel')
    def handleCancel(self, action):
        self.redirect_to_next_url(self.next_url)
        self.status = _('Deletion canceled.')

    @z3c.form.button.buttonAndHandler(_(u'Yes, delete'), name='delete')
    def handleDelete(self, action):
        self._handle_delete()

    def _handle_delete(self):
        self.redirect_to_next_url('parent', self.next_view_after_delete)
        self.status = _(
            '"${title}" deleted.',
            mapping=dict(
                title=icemac.addressbook.interfaces.ITitle(self.context)))
        self._do_delete()

    def _do_delete(self):
        name = zope.traversing.api.getName(self.context)
        parent = zope.traversing.api.getParent(self.context)
        del parent[name]


def delete_persons(address_book, ids):
    """Delete persons specified by their ID, but not users."""
    deleted = 0
    for name in list(ids):  # this list() call is needed as we might delete
                            # from the source of the ids
        ref_target = gocept.reference.interfaces.IReferenceTarget(
            zope.security.proxy.getObject(address_book[name]))
        if ref_target.is_referenced(recursive=False):
            # Persons which are referenced by a user can't be
            # deleted using this function. We check this here to
            # avoid getting an error.
            continue
        del address_book[name]
        deleted += 1
    return deleted


class PrefixGroup(z3c.form.group.Group):
    """Group which sets a prefix."""

    prefix = None  # to be set in subclass
    interface = None  # to be set in subclass

    @property
    def fields(self):
        fields = icemac.addressbook.interfaces.IEntity(self.interface)
        return z3c.form.field.Fields(
            *fields.getFieldValuesInOrder(), **dict(prefix=self.prefix))


class CloneObject(BaseView):
    """View class to clone an object and store the clone into the parent.

    Redirects to the default view of the clone."""

    def __call__(self):
        # clone object
        unsecure_context = zope.security.proxy.getObject(self.context)
        copier = zope.copypastemove.interfaces.IObjectCopier(unsecure_context)
        parent = self.context.__parent__
        new_name = copier.copyTo(parent)
        # redirect to clone
        self.request.response.redirect(self.url(parent[new_name]))
        zope.component.getUtility(
            z3c.flashmessage.interfaces.IMessageSource).send(
            _('"${object}" cloned.', mapping=dict(
                object=icemac.addressbook.interfaces.ITitle(self.context))))


def can_access(uri_part):
    """Create a button condition function to test whether the user can
    access the URL context/@@absolute_url/<uri_part>."""
    def can_access_form(form):
        traverser = zope.traversing.publicationtraverse.PublicationTraverser()
        try:
            view = traverser.traverseRelativeURL(
                form.request, form.context, uri_part)
        except (zope.security.interfaces.Unauthorized,
                zope.security.interfaces.Forbidden,
                LookupError):
            return False
        else:
            # we're assuming that view pages are callable
            # this is a pretty sound assumption
            if not zope.security.canAccess(view, '__call__'):
                return False
        return True
    return can_access_form



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: test_doctests.py 1183 2010-11-23 17:17:09Z icemac $

import icemac.addressbook.testing


def test_suite():
    return icemac.addressbook.testing.FunctionalDocFileSuite(
        # Caution: none of these tests can run as unittest!
        'adapter.txt',
        'address.txt',
        'addressbook.txt',
        'catalog.txt',
        'person.txt',
        'testing.txt',
        )



# -*- coding: utf-8 -*-
# Copyright (c) 2010 Michael Howitz
# See also LICENSE.txt

from icemac.addressbook.i18n import _
import icemac.addressbook.interfaces
import icemac.addressbook.sources
import stabledict
import zc.sourcefactory.basic
import zope.component


def tokenize(entity, field_name):
    "Convert an entity and a field_name into a unique string token."
    return "%s###%s" % (entity.name, field_name)


def untokenize(token):
    "Convert a token containing of entity and field name back to the objects."
    entity_name, field_name = token.split('###')
    entity = icemac.addressbook.interfaces.IEntity(entity_name)
    field = entity.getRawField(field_name)
    return entity, field


class ColumnSource(zc.sourcefactory.basic.BasicSourceFactory):
    "Columns in the person list."

    def getValues(self):
        # The main entities (the ones with default values on the
        # person) can be displayed in the person list.
        entities = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntities)
        for entity in entities.getMainEntities():
            # All fields, even the user defined ones, can be selected
            # as display columns.
            for field_name, field in entity.getRawFields():
                # We return a representation here which can be stored
                # in ZODB even though it is not so easy to compute a
                # title from it. But it woulod be more difficult to
                # adapt zope.preference to store this string instead
                # of field objects.
                yield tokenize(entity, field_name)

    def getTitle(self, value):
        entity, field = untokenize(value)
        # The titles might be message ids so allow to translate them.
        return _(u"${prefix} -- ${title}", mapping=dict(
            prefix=entity.title, title=field.title))


column_source = ColumnSource()


class AscDescSource(icemac.addressbook.sources.TitleMappingSource):
    _mapping = stabledict.StableDict(
        (('ascending', _(u'ascending (A-->Z)')),
         ('descending', _(u'descending (Z-->A)'))))

asc_des_csource = AscDescSource()



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: interfaces.py 783 2010-01-09 14:28:06Z icemac $

import z3c.menu.ready2go


class IMainMenu(z3c.menu.ready2go.ISiteMenu):
    """Main menu."""


class IAddMenu(z3c.menu.ready2go.IAddMenu):
    """Add menu."""



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
# $Id: root.py 783 2010-01-09 14:28:06Z icemac $

import icemac.addressbook.interfaces
import z3c.pagelet.browser
import zope.size.interfaces
import zope.traversing.browser.absoluteurl


class FrontPage(z3c.pagelet.browser.BrowserPagelet):
    """Pagelet for the front page."""

    def getAddressBooks(self):
        result = []
        for ab in self.context.values():
            if not icemac.addressbook.interfaces.IAddressBook.providedBy(ab):
                # only show address books
                continue
            url = zope.traversing.browser.absoluteurl.absoluteURL(
                ab, self.request)
            result.append(dict(
                title=ab.title,
                url=url,
                delete_url=url + '/@@delete_address_book.html',
                count=zope.size.interfaces.ISized(ab).sizeForDisplay()))
        return result



# -*- coding: utf-8 -*-
# Copyright (c) 2010 Michael Howitz
# See also LICENSE.txt

import zope.component
import zope.publisher.interfaces.browser
import zope.interface
import icemac.addressbook.export.interfaces


class BaseExporter(object):
    "Abstract base class for exporters which defines some convenience methods."

    zope.component.adapts(list,
                          zope.publisher.interfaces.browser.IBrowserRequest)
    zope.interface.implements(icemac.addressbook.export.interfaces.IExporter)

    # to be set in subclass
    file_extension = None
    mime_type = None
    title = None
    description = None

    def __init__(self, persons, request=None):
        self.persons = persons
        self.request = request

    def __eq__(self, other):
        return self.__class__ == other.__class__

    def export(self):
        raise NotImplementedError()



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

import icemac.addressbook.entities
import icemac.addressbook.orderstorage
import icemac.addressbook.testing
import unittest
import zope.component.testing


class EntitiesTests(object):

    entities_class = None

    def setUp(self):
        from icemac.addressbook.tests.stubs import setUpStubEntities
        zope.component.testing.setUp()
        setUpStubEntities(self, self.entities_class)
        # sort order
        order_store = icemac.addressbook.orderstorage.OrderStorage()
        order_store.add(self.cat.name, icemac.addressbook.interfaces.ENTITIES)
        order_store.add(
            self.kwack.name, icemac.addressbook.interfaces.ENTITIES)
        order_store.add(self.duck.name, icemac.addressbook.interfaces.ENTITIES)
        zope.component.provideUtility(
            order_store, icemac.addressbook.interfaces.IOrderStorage)
        self.entity_order = icemac.addressbook.entities.EntityOrder()
        zope.component.provideUtility(self.entity_order)

    def tearDown(self):
        zope.component.testing.tearDown()

    def test_getEntities(self):
        self.assertEqual(
            sorted([self.kwack, self.duck, self.cat]),
            sorted(self.entities.getEntities(sorted=False)))

    def test_getEntities_sorted(self):
        self.assertEqual(
            [self.cat, self.kwack, self.duck],
            self.entities.getEntities())

    def test_getEntitiesInOrder_changed_order(self):
        self.entity_order.up(self.duck)
        self.assertEqual(
            [self.cat, self.duck, self.kwack],
            self.entities.getEntities())


class TestEntities(EntitiesTests, unittest.TestCase):

    entities_class = icemac.addressbook.entities.Entities


class TestPersistentEntities(EntitiesTests, unittest.TestCase):

    entities_class = icemac.addressbook.entities.PersistentEntities


class TestEntities_getMainEntities(
    icemac.addressbook.testing.AddressBookFunctionalTestCase):

    def callFUT(self, sorted):
        import zope.component
        import icemac.addressbook.interfaces

        entities = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntities)
        return [x.title for x in entities.getMainEntities(sorted=sorted)]

    DEFAULT_ORDER = [u'person', u'postal address', u'phone number',
                     u'e-mail address', u'home page address']

    def test_default_order(self):
        # With unchanged sort order both variants return the same order:
        self.assertEqual(self.DEFAULT_ORDER, self.callFUT(True))
        self.assertEqual(self.DEFAULT_ORDER, self.callFUT(False))

    def test_changed_order(self):
        import zope.component
        import icemac.addressbook.interfaces

        order = zope.component.getUtility(
            icemac.addressbook.interfaces.IEntityOrder)
        phone_number = icemac.addressbook.interfaces.IEntity(
            icemac.addressbook.interfaces.IPhoneNumber)
        order.up(phone_number)
        self.assertEqual(
            [u'person', u'phone number', u'postal address', u'e-mail address',
             u'home page address'],
            self.callFUT(True))
        # Unordered variant still returns the default order:
        self.assertEqual(self.DEFAULT_ORDER, self.callFUT(False))


class TestEntityOrder(
    icemac.addressbook.testing.AddressBookFunctionalTestCase):

    def getEntity(self, iface_name):
        import icemac.addressbook.interfaces
        return icemac.addressbook.interfaces.IEntity(
            getattr(icemac.addressbook.interfaces, iface_name))

    @property
    def entity_order(self):
        return zope.component.getUtility(
            icemac.addressbook.interfaces.IEntityOrder)

    @property
    def unknown_entity(self):
        import icemac.addressbook.entities
        import icemac.addressbook.interfaces
        return icemac.addressbook.entities.Entity(
            u'', icemac.addressbook.interfaces.IEntities,
            'icemac.addressbook.entities.Entities')

    @property
    def minimal_entity(self):
        import icemac.addressbook.interfaces
        return icemac.addressbook.interfaces.IEntity(
            icemac.addressbook.interfaces.IEntities)

    def test_get_IPerson(self):
        self.assertEqual(1, self.entity_order.get(self.getEntity('IPerson')))

    def test_get_IKeyword(self):
        self.assertEqual(8, self.entity_order.get(self.getEntity('IKeyword')))

    def test_get_unknown_entity(self):
        self.assertRaises(KeyError, self.entity_order.get, self.unknown_entity)

    def test_get_minimal_entity(self):
        # When the entity has no name, a KeyError is raised, too.
        self.assertRaises(KeyError, self.entity_order.get, self.minimal_entity)

    def test_isFirst_first(self):
        self.assertTrue(
            self.entity_order.isFirst(self.getEntity('IAddressBook')))

    def test_isFirst_not_first(self):
        self.assertFalse(
            self.entity_order.isFirst(self.getEntity('IPhoneNumber')))

    def test_isFirst_unknown_entity(self):
        self.assertRaises(
            KeyError, self.entity_order.isFirst, self.unknown_entity)

    def test_isFirst_minimal_entity(self):
        self.assertRaises(
            KeyError, self.entity_order.isFirst, self.minimal_entity)

    def test_isLast_last(self):
        self.assertTrue(
            self.entity_order.isLast(self.getEntity('IKeyword')))

    def test_isLast_not_last(self):
        self.assertFalse(
            self.entity_order.isLast(self.getEntity('IPhoneNumber')))

    def test_isLast_unknown_entity(self):
        self.assertRaises(
            KeyError, self.entity_order.isLast, self.unknown_entity)

    def test_isLast_minimal_entity(self):
        self.assertRaises(
            KeyError, self.entity_order.isLast, self.minimal_entity)

    def test___iter__(self):
        self.assertEqual(['IcemacAddressbookAddressbookAddressbook',
                          'IcemacAddressbookPersonPerson',
                          'IcemacAddressbookPersonPersondefaults',
                          'IcemacAddressbookAddressPostaladdress',
                          'IcemacAddressbookAddressPhonenumber',
                          'IcemacAddressbookAddressEmailaddress',
                          'IcemacAddressbookAddressHomepageaddress',
                          'IcemacAddressbookFileFileFile',
                          'IcemacAddressbookKeywordKeyword'],
                         [x for x in self.entity_order])

    def test_up_w_o_delta(self):
        person = self.getEntity('IPerson')
        self.assertEqual(1, self.entity_order.get(person))
        self.entity_order.up(person)
        self.assertEqual(0, self.entity_order.get(person))

    def test_up_w_delta(self):
        hp = self.getEntity('IHomePageAddress')
        self.assertEqual(6, self.entity_order.get(hp))
        self.entity_order.up(hp, 3)
        self.assertEqual(3, self.entity_order.get(hp))

    def test_up_too_much(self):
        person = self.getEntity('IPerson')
        self.assertEqual(1, self.entity_order.get(person))
        self.assertRaises(ValueError, self.entity_order.up, person, 2)

    def test_down_w_o_delta(self):
        person = self.getEntity('IPerson')
        self.assertEqual(1, self.entity_order.get(person))
        self.entity_order.down(person)
        self.assertEqual(2, self.entity_order.get(person))

    def test_down_w_delta(self):
        ab = self.getEntity('IAddressBook')
        self.assertEqual(0, self.entity_order.get(ab))
        self.entity_order.down(ab, 3)
        self.assertEqual(3, self.entity_order.get(ab))

    def test_down_too_much(self):
        person = self.getEntity('IPerson')
        self.assertEqual(1, self.entity_order.get(person))
        self.assertRaises(ValueError, self.entity_order.down, person, 8)

    def test_other_address_book(self):
        # IEntityStorage always accesses the current address book as defined
        # by the setSite hook.
        import icemac.addressbook.testing
        import zope.site.hooks

        ab2 = icemac.addressbook.testing.create_addressbook(
            'ab2', parent=self.layer.getRootFolder())

        person = self.getEntity('IPerson')
        self.assertEqual(1, self.entity_order.get(person))
        self.entity_order.down(person)
        self.assertEqual(2, self.entity_order.get(person))

        zope.site.hooks.setSite(ab2)
        self.assertEqual(1, self.entity_order.get(person))

    def test_outside_address_book(self):
        # Outside an address book a ComponentLookupError is raised:
        import zope.component
        import zope.site.hooks

        zope.site.hooks.setSite(self.old_site)
        self.assertRaises(
            zope.component.ComponentLookupError,
            self.entity_order.get, self.getEntity('IPerson'))


class TestEntityAdapters(unittest.TestCase):

    def setUp(self):
        from icemac.addressbook.tests.stubs import setUpStubEntities
        zope.component.testing.setUp()
        setUpStubEntities(self, icemac.addressbook.entities.Entities)
        zope.component.provideAdapter(
            icemac.addressbook.entities.entity_by_name)
        zope.component.provideAdapter(
            icemac.addressbook.entities.entity_by_interface)
        zope.component.provideAdapter(
            icemac.addressbook.entities.entity_by_obj)

    def tearDown(self):
        zope.component.testing.tearDown()

    # no adapter
    def test_unknown_type(self):
        self.assertRaises(
            TypeError, icemac.addressbook.interfaces.IEntity, None)

    # adaption from string
    def test_unknown_name(self):
        self.assertRaises(
            ValueError, icemac.addressbook.interfaces.IEntity, 'asdf')

    def test_class_name(self):
        self.assertRaises(
            ValueError, icemac.addressbook.interfaces.IEntity,
            'icemac.addressbook.tests.stubs.Duck')

    def test_known_name(self):
        self.assertEqual(
            self.duck, icemac.addressbook.interfaces.IEntity(
                'IcemacAddressbookTestsStubsDuck'))

    # adaption from interface
    def test_unknown_interface(self):
        from icemac.addressbook.tests.stubs import IDog
        entity = icemac.addressbook.interfaces.IEntity(IDog)
        self.assert_(None is entity.title)
        self.assert_(IDog is entity.interface)
        self.assert_(None is entity.class_name)

    def test_known_interface(self):
        from icemac.addressbook.tests.stubs import IKwack
        self.assertEqual(
            self.kwack, icemac.addressbook.interfaces.IEntity(IKwack))

    # adaption from object
    def test_unknown_object(self):
        from persistent import Persistent
        obj = Persistent()
        self.assertRaises(
            ValueError, icemac.addressbook.interfaces.IEntity, obj)

    def test_known_object(self):
        from icemac.addressbook.tests.stubs import Kwack
        self.assertEqual(
            self.kwack, icemac.addressbook.interfaces.IEntity(Kwack()))

    # title
    def test_getTitle(self):
        self.assertEqual(
            u'Kwack', icemac.addressbook.interfaces.IEntity(
                'IcemacAddressbookTestsStubsKwack').title)



# -*- coding: latin-1 -*-
# Copyright (c) 2008-2010 Michael Howitz
# See also LICENSE.txt
import icemac.addressbook.generations.utils

# needed to make sure this package still exists and the instances get
# converted to the new class path
import zope.app.authentication

a = zope.app.authentication


def evolve(context):
    """Updates the authentication utility.
    """
    icemac.addressbook.generations.utils.update_address_book_infrastructure(
        context)



import unittest


class Test_default_attrib_name_to_entity(unittest.TestCase):

    def setUp(self):
        from icemac.addressbook.tests.stubs import setUpStubEntities
        import icemac.addressbook.entities
        import zope.component.testing

        zope.component.testing.setUp()
        setUpStubEntities(self, icemac.addressbook.entities.Entities)

    def tearDown(self):
        import zope.component.testing
        zope.component.testing.tearDown()

    def callFUT(self, name):
        from icemac.addressbook.address import default_attrib_name_to_entity
        return default_attrib_name_to_entity(name)

    def test_unknown_name(self):
        self.assertRaises(ValueError, self.callFUT, 'foo')

    def test_known_name(self):
        self.assertEqual(self.duck, self.callFUT('default_duck'))



# -*- coding: utf-8 -*-
# Copyright (c) 2010 Michael Howitz
# See also LICENSE.txt

import icemac.addressbook.browser.resource
import z3c.preference.browser


class EditForm(icemac.addressbook.browser.base.BaseEditFormWithCancel,
               z3c.preference.browser.EditForm):
    """Preference EditForm which uses addressbook's form CSS."""

    next_url = 'site'
    __init__ = z3c.preference.browser.EditForm.__init__
    fields = z3c.preference.browser.EditForm.fields

    def update(self):
        icemac.addressbook.browser.resource.form_css.need()
        super(EditForm, self).update()



# -*- coding: utf-8 -*-
# Copyright (c) 2010 Michael Howitz
# See also LICENSE.txt

import icemac.addressbook.interfaces
import zope.app.appsetup.bootstrap
import zope.preference.default
import zope.preference.interfaces
import icemac.addressbook.utils


def add(address_book):
    """Add a default preferences provider to the address book."""
    # Add a default preferences utility on the address book site.
    if icemac.addressbook.utils.utility_locally_registered(
            address_book,
            zope.preference.interfaces.IDefaultPreferenceProvider):
        default_prefs = zope.component.getUtility(
            zope.preference.interfaces.IDefaultPreferenceProvider)
    else:
        default_prefs = zope.app.appsetup.bootstrap.ensureUtility(
            address_book,
            zope.preference.interfaces.IDefaultPreferenceProvider, '',
            zope.preference.default.DefaultPreferenceProvider)

    # Set the defaults for the person list.
    personList = default_prefs.getDefaultPreferenceGroup('personList')
    personList.columns = []
    # The default columns are person last name and person first name.
    person_entity = icemac.addressbook.interfaces.IEntity(
        icemac.addressbook.interfaces.IPerson)
    personList.columns.append(
        icemac.addressbook.preferences.sources.tokenize(
            person_entity, 'last_name'))
    personList.columns.append(
        icemac.addressbook.preferences.sources.tokenize(
            person_entity, 'first_name'))
    personList._p_changed = True
    # The default sort column is person last name.
    personList.order_by = icemac.addressbook.preferences.sources.tokenize(
        person_entity, 'last_name')
    # The default sort direction is ascending
    personList.sort_direction = 'ascending'
    personList.batch_size = 20



# -*- coding: utf-8 -*-
# Copyright (c) 2009-2010 Michael Howitz
# See also LICENSE.txt

import icemac.addressbook.entities
import icemac.addressbook.interfaces
import unittest
import zope.component.testing
import zope.interface
import zope.schema


class IDummy(zope.interface.Interface):

    dummy = zope.schema.Text(title=u'dummy')
    dummy2 = zope.schema.Text(title=u'dummy2')


class Dummy(object):
    pass


class TestEntity(unittest.TestCase):

    def setUp(self):
        self.entity = icemac.addressbook.entities.Entity(
            u'Dummy', IDummy, 'icemac.addressbook.tests.test_entity.Dummy')

        self.user_field = icemac.addressbook.entities.Field()
        self.user_field.__name__ = 'Field#1'
        self.user_field.type = 'TextLine'
        zope.component.provideAdapter(
            icemac.addressbook.entities.FieldAdapterFactory(self.user_field),
            adapts=(icemac.addressbook.interfaces.IEntity, IDummy),
            provides=icemac.addressbook.interfaces.IField,
            name=self.user_field.__name__)
        self.schemaized_field = (
            icemac.addressbook.entities.user_field_to_schema_field(
                self.user_field))

    def tearDown(self):
        zope.component.testing.tearDown()

    def test_name(self):
        self.assertEqual('IcemacAddressbookTestsTestEntityDummy',
                         self.entity.name)

    def test_name_no_class_name_set(self):
        self.assertRaises(
            ValueError, getattr,
            icemac.addressbook.entities.Entity(None, IDummy, None), "name")

    def test_getRawFields(self):
        self.assertEqual([('dummy', IDummy['dummy']),
                          ('dummy2', IDummy['dummy2']),
                          ('Field#1', self.user_field)],
                         list(self.entity.getRawFields()))

    def test_getFieldsInOrder(self):
        self.assertEqual([('dummy', IDummy['dummy']),
                          ('dummy2', IDummy['dummy2']),
                          ('Field#1', self.schemaized_field)],
                         list(self.entity.getFieldsInOrder()))

    def test_getFieldValuesInOrder(self):
        self.assertEqual(
            [IDummy['dummy'], IDummy['dummy2'], self.schemaized_field],
            self.entity.getFieldValuesInOrder())

    def test_getField_unknown_field(self):
        self.assertRaises(KeyError, self.entity.getField, 'asdf')

    def test_getField_schema_field(self):
        self.assertEqual(IDummy['dummy2'], self.entity.getField('dummy2'))

    def test_getField_user_field(self):
        self.assertEqual(
            self.schemaized_field, self.entity.getField('Field#1'))

    def test_getRawField_unknown_field(self):
        self.assertRaises(KeyError, self.entity.getRawField, 'asdf')

    def test_getRawField_schema_field(self):
        self.assertEqual(IDummy['dummy2'], self.entity.getRawField('dummy2'))

    def test_getRawField_user_field(self):
        self.assertEqual(
            self.user_field, self.entity.getRawField('Field#1'))

    def test_getClass(self):
        self.assertEqual(Dummy, self.entity.getClass())

    def test_getClass_no_class_set(self):
        e = icemac.addressbook.entities.Entity(None, IDummy, None)
        self.assertRaises(ValueError, e.getClass)

    def test_tagged_values(self):
        e = icemac.addressbook.entities.Entity(
            u'Dummy', IDummy, 'Dummy', a=1, b='asdf')
        self.assertEqual(dict(a=1, b='asdf'), e.tagged_values)

    def test_tagged_values_returns_copy(self):
        # Tagged values not modifyable by modifying the returned dict.
        e = icemac.addressbook.entities.Entity(
            u'Dummy', IDummy, 'Dummy', a=1, b='asdf')
        e.tagged_values['a'] = 2
        self.assertEqual(dict(a=1, b='asdf'), e.tagged_values)
